{-------------------------------------------------------------------
 - UNIT:   MYGROUP.PAS                                             -
 - TARGET: Delphi 1 (may work in Delphi 2 & 3)                     -
 - AUTHOR: Programmed by Ron Marx, October 1997                    -
 - EMAIL:  Send any comments/change requests/etc. to               -
 -         marxron@ix.netcom.com                                   -
 -------------------------------------------------------------------
 - !INSTALLATION!:  In Delphi, go to Options|Install Component and -
 - add MYGROUP.PAS or MYGROUP.DCU.  This will add the component to -
 - the Data Controls page of your component palette.               -
 -------------------------------------------------------------------
 - PROPERTIES:  There are three new properties which were not      -
 - inherited from the Delphi version of TDBRadioGroup.             -
 -                                                                 -
 - CaptionFont - Allows you to change the caption font.            -
 -                                                                 -
 - SearchFirstChar: When set to true it will set the focus to the  -
 - first choice which matches the character you just typed in.  It -
 - is case sensitive.                                              -
 -                                                                 -
 - ShowFrame: When set to true it shows the frame.                 -
 -------------------------------------------------------------------
 - WARNING: This software is provided as is.  No warranty is given -
 - by the author, expressed or implied.  Use this software at your -
 - own risk.  The author assumes no responsibility for any damage  -
 - from the use of this software.                                  -
 -                                                                 -
 - NOTE:  This software is freeware which means that it is free    -
 - for use and distribution.                                       -
 -------------------------------------------------------------------}

unit MyGroup;

interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, Forms,
  Graphics, Menus, StdCtrls, ExtCtrls, DB, DBTables, Mask, Buttons, Dialogs;

type
  TMyDBRadioGroup = class(TCustomRadioGroup)
  private
    FSearchFirstChar: Boolean;
    FShowFrame: Boolean;
    FIndex: Integer;
    FDataLink: TFieldDataLink;
    FValue: PString;
    FValues: TStrings;
    FOnChange: TNotifyEvent;
    FCaptionFont: TFont;
    FSelfChangingCaptionFont: Boolean;
    procedure SetSearchFirstChar(A: Boolean);
    procedure SetShowFrame(A: Boolean);
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    procedure SetCaptionFont(Value: TFont);
    procedure CaptionFontChanged(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    function GetValue: string;
    function GetButtonValue(Index: Integer): string;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure SetValue(const Value: string);
    procedure SetItems(Value: TStrings);
    procedure SetValues(Value: TStrings);
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMFontChanged(var message: TMessage); message CM_FONTCHANGED;
    procedure CMParentFontChanged(var message: TMessage); message CM_PARENTFONTCHANGED;
  protected
    procedure Change; dynamic;
    procedure Click; override;
    procedure KeyPress(var Key: Char); override;
    function CanModify: Boolean; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property DataLink: TFieldDataLink read FDataLink;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateMyRadioGroup(Sender: TObject);
    procedure Paint; override;
    property Field: TField read GetField;
    property ItemIndex;
    property Value: string read GetValue write SetValue;
  published
    property Align;
    property Caption;
    property CaptionFont: TFont read FCaptionFont write SetCaptionFont;
    property Color;
    property Columns;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Items write SetItems;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property SearchFirstChar: Boolean read FSearchFirstChar write SetSearchFirstChar default True;
    property ShowFrame: Boolean read FShowFrame write SetShowFrame default True;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Values: TStrings read FValues write SetValues;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Data Controls', [TMyDBRadioGroup]);
end;

constructor TMyDBRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  FValue := NullStr;
  FValues := TStringList.Create;
  FCaptionFont := TFont.Create;
  FCaptionFont.OnChange := CaptionFontChanged;
  FSearchFirstChar := True;
  FShowFrame := True;
  FIndex := 0;
end;

destructor TMyDBRadioGroup.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  DisposeStr(FValue);
  FValues.Free;
  inherited Destroy;
end;

procedure TMyDBRadioGroup.SetSearchFirstChar(A: Boolean);
begin
  if FSearchFirstChar <> A then
    begin
      FSearchFirstChar := A;
      Invalidate;
    end;
end;

procedure TMyDBRadioGroup.SetShowFrame(A: Boolean);
begin
  if FShowFrame <> A then
    begin
      FShowFrame := A;
      Invalidate;
    end;
end;

procedure TMyDBRadioGroup.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TMyDBRadioGroup.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    Value := FDataLink.Field.Text
  else
    Value := EmptyStr;
end;

procedure TMyDBRadioGroup.CaptionFontChanged(Sender: TObject);
begin
  if (not FSelfChangingCaptionFont) and not (csLoading in ComponentState) then
    ParentFont := False;
end;

procedure TMyDBRadioGroup.UpdateData(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    FDataLink.Field.Text := Value;
end;

function TMyDBRadioGroup.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TMyDBRadioGroup.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TMyDBRadioGroup.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TMyDBRadioGroup.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TMyDBRadioGroup.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TMyDBRadioGroup.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TMyDBRadioGroup.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TMyDBRadioGroup.GetValue : string;
begin
  Result := FValue^;
end;

function TMyDBRadioGroup.GetButtonValue(Index: Integer): string;
begin
  if (Index < FValues.Count) and (FValues[Index] <> '') then
    Result := FValues[Index]
  else if (Index < Items.Count) then
    Result := Items[Index]
  else
    Result := '';
end;

procedure TMyDBRadioGroup.SetValue (const Value: string);
var
  I : Integer;
begin
  AssignStr(FValue, Value);
  if (ItemIndex < 0) or (GetButtonValue(ItemIndex) <> Value) then
    begin
      if (ItemIndex >= 0) then ItemIndex := -1;
        for I := 0 to ControlCount - 1 do
          begin
            if GetButtonValue(I) = Value then
              begin
                ItemIndex := I;
                Break;
              end;
          end;
        Change;
    end;
end;

procedure TMyDBRadioGroup.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    if ItemIndex >= 0 then
      TRadioButton(Controls[ItemIndex]).SetFocus
    else
      TRadioButton(Controls[0]).SetFocus;
    raise;
  end;
  inherited;
end;

procedure TMyDBRadioGroup.Click;
begin
  inherited Click;
  if ItemIndex >= 0 then
    Value := GetButtonValue(ItemIndex);
  if FDataLink.Editing then
    FDataLink.Modified;
end;

procedure TMyDBRadioGroup.SetItems(Value: TStrings);
begin
  Items.Assign(Value);
  DataChange(Self);
end;

procedure TMyDBRadioGroup.SetValues(Value: TStrings);
begin
  FValues.Assign(Value);
  DataChange(Self);
end;

procedure TMyDBRadioGroup.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TMyDBRadioGroup.KeyPress(var Key: Char);
var
  Index : Integer;
begin
  inherited KeyPress(Key);
  case Key of
    #8, ' ':
      FDataLink.Edit;
    #27:
      FDataLink.Reset;
  end;
  if SearchFirstChar then
    begin
      { code which allows you to type first character of an item in the item list }
      if Self.Items.Count > 0 then
        begin
          for Index := FIndex to (Self.Items.Count - 1) do
            begin
              if Key = Copy(Self.Items[Index], 0, 1) then
                begin
                  Self.ItemIndex := Index;
                  TRadioButton(Controls[ItemIndex]).SetFocus;
                  if FIndex = Self.Items.Count - 1 then
                    FIndex := 0
                  else
                    FIndex := Index + 1;
                  Exit;
                end;
            end;
          FIndex := 0;
        end;
    end;
end;

function TMyDBRadioGroup.CanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

procedure TMyDBRadioGroup.SetCaptionFont(Value: TFont);
begin
  FCaptionFont.Assign(Value);
  UpdateMyRadioGroup(Self);
end;

procedure TMyDBRadioGroup.CMFontChanged(var message: TMessage);
begin
  inherited;
end;

procedure TMyDBRadioGroup.CMParentFontChanged(var message: TMessage);
begin
  inherited;
  if ParentFont then
    FSelfChangingCaptionFont := True;
end;

procedure TMyDBRadioGroup.UpdateMyRadioGroup(Sender: TObject);
begin
  Paint;
end;

procedure TMyDBRadioGroup.Paint;
var
  H: Integer;
  R: TRect;
  C: array [Byte] of Char;
  CLen: Integer;
begin
  with Canvas do
    begin
      Font := Self.CaptionFont;
      if FShowFrame then
        begin
          H := TextHeight('0');
          R := Rect(0, H div 2 - 1, Width, Height);
          if Ctl3D then
            begin
              Inc(R.Left);
              Inc(R.Top);
              Brush.Color := clBtnHighlight;
              FrameRect(R);
              OffsetRect(R, -1, -1);
              Brush.Color := clBtnShadow;
            end
          else
            Brush.Color := clWindowFrame;
          FrameRect(R);
        end;
      StrPCopy(C, Text);
      if C[0] <> #0 then
        begin
          StrPCopy(C, Text);
          CLen := StrLen(C);
          R := Rect(8, 0, 0, H);
          DrawText(Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE or DT_CALCRECT);
          Brush.Color := Color;
          DrawText(Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE);
        end;
    end;
end;

end.
